from django.db import models


class OrdinalField(models.FloatField):

    def __init__(self, *args, **kwargs):
        kwargs['null'] = kwargs.get('null', False)
        kwargs['default'] = kwargs.get('default', 0)
        super(OrdinalField, self).__init__(*args, **kwargs)


class IntegerRangeField(models.IntegerField):

    def __init__(self, verbose_name=None, name=None, min_value=None, max_value=None, **kwargs):
        self.min_value, self.max_value = min_value, max_value
        models.IntegerField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'min_value': self.min_value, 'max_value': self.max_value}
        defaults.update(kwargs)
        return super(IntegerRangeField, self).formfield(**defaults)



import datetime

from django.template import Template, Context
from django.test import TestCase, RequestFactory

from models import TestDateTimeModel


class TagsTestCase(TestCase):
    def setUp(self):
        self.request_factory = RequestFactory()

    def test_url_replace(self):
        request = self.request_factory.get('/test_url_replace?foo=bar&page=1')
        template = Template("{% load common %} <a href=\"?{% url_replace request 'page' '2' %}\"></a>")  # nopep8
        response = template.render(Context({'request': request}))
        self.assertIn('page=2', response)
        self.assertNotIn('page=1', response)
        self.assertIn('foo=bar', response)


class ModelTestCase(TestCase):

    def test_datetime_model_mixin(self):
        test_dt = TestDateTimeModel()

        self.assertIsNone(test_dt.created_at)
        self.assertIsNone(test_dt.updated_at)

        test_dt.save()

        self.assertIsNotNone(test_dt.created_at)
        self.assertIsNotNone(test_dt.updated_at)
        self.assertIsInstance(test_dt.created_at, datetime.datetime)
        self.assertIsInstance(test_dt.updated_at, datetime.datetime)



from django import template

register = template.Library()


@register.simple_tag
def url_replace(request, field, value):
    '''
    This is handy for generating links in templates where you want to change
    one variable but keep all existing GET request vars.

    Example:
    <a href="?{% url_replace request 'page' page_obj.next_page_number %}">next</a>

    :param request:
    :param field:
    :param value:
    :return:
    '''
    dict_ = request.GET.copy()

    dict_[field] = value

    return dict_.urlencode()



from collections import OrderedDict

from django.db import models
from django.db.models import QuerySet


class DateTimeModelMixin(models.Model):
    created_at = models.DateTimeField(auto_now_add=True, verbose_name='Created')
    updated_at = models.DateTimeField(auto_now=True, verbose_name='Updated')

    class Meta:
        abstract = True


def mptt_build_tree(queryset, children_key):
    '''
    :param queryset: the entire tree, or portion of tree, you're trying to construct
    :param children_key: key to store children, can't be a field name that's already on the model
    :return: QuerySet, with children (also QuerySets) properly nested to each level present
    '''
    nodes = OrderedDict()
    nodes_children = OrderedDict()

    for row in queryset:
        nodes[row.id] = row
        if nodes_children.get(row.parent_id):
            nodes_children[row.parent_id]._result_cache.append(row)
        else:
            nodes_children[row.parent_id] = QuerySet()
            nodes_children[row.parent_id]._result_cache = [row]

    for k, node in nodes.iteritems():
        # Try to find the parent so we can attach the child nodes
        setattr(node, children_key, QuerySet())
        children = getattr(node, children_key)
        children._result_cache = []
        if nodes.get(node.parent_id):
            children = getattr(nodes[node.parent_id], children_key)
            children._result_cache.append(node)

    # Build out root nodes for the base level of the list
    root_nodes = QuerySet()
    root_nodes._result_cache = []
    for node_id, node in nodes.iteritems():
        if node.parent_id is None:
            root_nodes._result_cache.append(node)

    return root_nodes


class MPTTDescendantsTreeMixin(object):

    def get_descendants_as_tree(self, children_key='children'):
        return mptt_build_tree(self.get_descendants(include_self=True), children_key=children_key)[0]





class NotDeletableAdminMixin(object):

    def get_actions(self, request):
        actions = super(NotDeletableAdminMixin, self).get_actions(request)
        actions.pop('delete_selected', None)
        return actions

    def has_delete_permission(self, request, obj=None):
        return False


class NotAddableAdminMixin(object):

    def has_add_permission(self, request):
        return False



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='fh-django-common',
    version='0.2.10',
    packages=['common'],
    include_package_data=True,
    description='A Django app to provide common functionality in Django projects',
    long_description=README,
    url='https://gitlab.com/futurehaus/django-common',
    tests_require=['Django'],
    test_suite='runtests.main',
)
